
import 'package:dio/dio.dart';
import 'package:labs_petrushin/Presentation/home_page/home_page.dart';
import 'package:labs_petrushin/data/dtos/foods_dto.dart';
import 'package:labs_petrushin/data/mappers/food_mapper.dart';
import 'package:labs_petrushin/repositories/api_interface.dart';
import 'package:pretty_dio_logger/pretty_dio_logger.dart';

import '../domain/models/home.dart';

class FoodRepository extends ApiInterface {
  static final Dio _dio = Dio()
      ..interceptors.add(
        PrettyDioLogger(
          requestHeader: true,
          requestBody: true,
        ));

  // 91xPcWwfSGljSRMuoS8IH0GP4hM9QqwwtgSzqJMw
  static const String _baseUrl = 'https://api.nal.usda.gov';

  @override
  Future<HomeData?> loadData({String? q, int page = 1, int pageSize = 5}) async {
    try {
      final String searchQuery = q ?? 'food'; // Используем 'food' как значение по умолчанию
      final String url = '$_baseUrl/fdc/v1/foods/search?api_key=91xPcWwfSGljSRMuoS8IH0GP4hM9QqwwtgSzqJMw&query=$searchQuery&pageSize=10';

      final Response<dynamic> response = await _dio.get<Map<dynamic, dynamic>>(url,
      queryParameters: {
        'page[number]': page,
        'page[size]': pageSize,
      }
      );

      final FoodsDto dto = FoodsDto.fromJson(response.data as Map<String, dynamic>);
      // final List<CardData>? data = dto.foods?.map((e) => e.toDomain()).toList();
      final HomeData data = dto.toDomain();
      return data;
    } on DioException catch (e) {
      // err
      return null;
    }
  }
}